@@ -134,17 +134,23 @@ class Group(ObjectsBase):
         self._uin_name_map = {}  # 群成员昵称到uin的映射
         self._id = UniqueIds.alloc(code, UniqueIds.T_GRP)
 
+    def update(self, **kw):
+        for key, val in kw.items():
+            setattr(self, key, val)
+
     def set_group_detail(self, data):
         """ 设置组详细信息, 包括群成员信息, 等.
         """
         for kw in data.get("minfo", []):
-            tmp = GroupMInfo(**kw)
-            self._uin_name_map[tmp.nick] = tmp.uin
-            self._uin_map[tmp.uin] = tmp
+            nick, uin = kw.get("nick"), kw.get("uin")
+            if nick is not None:
+                tmp = GroupMInfo(**kw)
+                self._uin_name_map[tmp.nick] = tmp.uin
+                self._uin_map[tmp.uin] = tmp
 
         for item in data.get("cards", []):
             uin = item.get("muin")
-            if uin in self._uin_map:
+            if uin in self._uin_map and item.get("card") is not None:
                 self._uin_map[uin].card = item.get("card")
                 self._uin_name_map[item.get("card")] = uin
             else:
@@ -260,18 +266,27 @@ class GroupList(ObjectsBase):
         self._gcode_map = {}
         self._gid_gcode_map = {}
         self._gcode_name_map = {}
+        self.update(data)
+
+    def update(self, data):
         for kw in data.get("gnamelist", []):
-            group = Group(**kw)
             gcode = kw.get("code")
-            self._gcode_map[gcode] = group
-            self._gid_gcode_map[kw.get("gid")] = gcode
-            self._gcode_name_map[kw.get("name")] = gcode
+            if gcode not in self._gcode_map:
+                group = Group(**kw)
+                self._gcode_map[gcode] = group
+                self._gid_gcode_map[kw.get("gid")] = gcode
+                self._gcode_name_map[kw.get("name")] = gcode
+            else:
+                self._gcode_map[gcode].update(**kw)
         self.gmasklist = data.get("gmasklist", [])
-        self.gnamelist = [Group(**kw) for kw in data.get("gnamelist", [])]
         self.gmarklist = data.get("gmarklist", [])
 
+    @property
+    def gnamelist(self):
+        return self._gcode_map.values()
+
     def __repr__(self):
-        return str([x.name for x in self.gnamelist])
+        return str([x.name for x in self._gcode_map.values()])
 
     def __unicode__(self):
         return self.__repr__().decode("utf-8")
@@ -289,7 +304,7 @@ class GroupList(ObjectsBase):
         return [x for x in self._gcode_map.values()]
 
     def get_gcodes(self):
-        return [x.code for x in self.gnamelist]
+        return [x.code for x in self._gcode_map.values()]
 
     def find_group(self, gcode):
         return self._gcode_map.get(gcode)
@@ -399,9 +414,18 @@ class DiscuList(ObjectsBase):
     def __init__(self, data):
         self._did_map = {}
         self._did_name_map = {}
+
+        self.update(data)
+
+    def update(self, data):
         for item in data.get("dnamelist"):
-            self._did_name_map[item["name"]] = item["did"]
-            self._did_map[item["did"]] = Discu(item["did"], item["name"])
+            did = item["did"]
+            if did not in self._did_map:
+                self._did_name_map[item["name"]] = item["did"]
+                self._did_map[did] = Discu(item["did"], item["name"])
+            else:
+                self._did_map[did].did = did
+                self._did_map[did].name = item["name"]
 
     @property
     def dids(self):
@@ -494,6 +518,10 @@ class FriendInfo(ObjectsBase):
         self.account = account
         self._id = UniqueIds.alloc(uin, UniqueIds.T_FRI)
 
+    def update(self, **kwargs):
+        for key, val in kwargs.items():
+            setattr(self, key, val)
+
     def set_markname(self, markname):
         self.markname = markname
 
@@ -526,11 +554,17 @@ class Friends(ObjectsBase):
         self._uin_map = {}
         self._name_map = {}
         self._mark_uin_map = {}
+        self.update(data)
 
+    def update(self, data):
         for item in data.get("info", {}):
-            info = FriendInfo(**item)
-            self._uin_map[info.uin] = info
-            self._name_map[info.nick] = info.uin
+            uin = item["uin"]
+            if uin not in self._uin_map:
+                info = FriendInfo(**item)
+                self._uin_map[info.uin] = info
+                self._name_map[info.nick] = info.uin
+            else:
+                self._uin_map[uin].update(**item)
 
         for item in data.get("friends", []):
             uin = item.get("uin")
